import 'package:blox/blox.dart';
import 'package:blox/constants/constants.dart';
import 'package:blox/constants/package_config.dart';
import 'package:blox/models/block.dart';
import 'package:blox/models/color_block.dart';
import 'package:blox/models/image_block.dart';
import 'package:blox/models/image_block_attr.dart';
import 'package:blox/models/text_block.dart';
import 'package:blox/models/text_block_attr.dart';
import 'package:blox/widgets/add_block.dart';
import 'package:blox/widgets/custom_reorderable_listview.dart';
import 'package:blox/widgets/blocks/text_editor.dart';
import 'package:flare_flutter/flare_actor.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

class BloxEditor extends StatefulWidget {
  /// If the Mouse come in hover of the addblock button it will be open if you set this parameter as true
  final bool addBlockButtonOpenOnHover;

  /// The Write can't delete block if you set it false.
  final bool canDeleteBlocks;

  /// use the BloxImageEditor for this variable to manage the pictures in editor
  final BloxImageEditor Function(ImageBlock imageBlock) imageEditor;

  /// Control the BloxEditor, get the result with [BloxController]
  final BloxController controller;

  /// Equal FA/EN set the color picker hints (آبی/Blue)
  final String colorPickerHintLang;

  /// The Border radius of blocks
  final double borderRadius;

  const BloxEditor(
      {Key key,
      this.addBlockButtonOpenOnHover = false,
      this.canDeleteBlocks = true,
      @required this.imageEditor,
      @required this.controller,
      this.colorPickerHintLang = "FA",
      this.borderRadius = 16})
      : super(key: key);

  @override
  _BloxEditorState createState() => _BloxEditorState();
}

class _BloxEditorState extends State<BloxEditor> {
  List _list = [];

  int _addButtonHoveredIndex = -1;
  List<Map<String, dynamic>> finaleResult = [];

  @override
  void initState() {
    super.initState();
    if (widget.controller != null) _list = widget.controller.bloxItems;
  }

  SystemMouseCursor cursor = SystemMouseCursors.basic;

  @override
  Widget build(BuildContext context) {
    return MouseRegion(
      cursor: cursor,
      child: CustomReorderableListView(
        onDragBegin: () {
          setState(() {
            cursor = SystemMouseCursors.grabbing;
          });
        },
        onDragDone: () {
          setState(() {
            cursor = SystemMouseCursors.basic;
          });
        },
        onReorder: (oldIndex, newIndex) {
          if (_list.length > 0) {
            var oldItem = _list[oldIndex];
            setState(() {
              _list.removeAt(oldIndex);
              if (_list.length < newIndex) {
                _list.insert(newIndex - 1, oldItem);
              } else {
                _list.insert(newIndex, oldItem);
              }
              if (widget.controller != null)
                widget.controller.bloxItems = _list;
            });
          }
          return true;
        },
        padding: EdgeInsets.all(8),
        children: <Widget>[
          for (int i = 0; i < _list.length; i++) ...[
            /// TextBlock Editor Section
            if (_list[i] is TextBlock) ...[
              _buildListItem(
                  i,
                  BloxTextEditor(
                    colorPickerHintLang: widget.colorPickerHintLang,
                    textSize: (_list[i] as TextBlock).style.fontSize,
                    fontFamily: (_list[i] as TextBlock).style.fontFamily,
                    align: (_list[i] as TextBlock).style.align,
                    color: (_list[i] as TextBlock).style.color,
                    multiLine: (_list[i] as TextBlock).style.multiline,
                    text: (_list[i] as TextBlock).text,
                    onChange: (value, fontSize, fontFamily, color, align) {
                      (_list[i] as TextBlock).text = value;
                      (_list[i] as TextBlock).style.fontSize = fontSize;
                      (_list[i] as TextBlock).style.fontFamily = fontFamily;
                      (_list[i] as TextBlock).style.color = color;
                      (_list[i] as TextBlock).style.align = align;
                    },
                  ))
            ],

            /// ImageBlock Editor Section
            if (_list[i] is ImageBlock) ...[
              _buildListItem(i, widget.imageEditor(_list[i])),
            ],

            /// Add More Blocks Here
          ],
          if (_list.length == 0) ...[
            Container(
              width: 400,
              height: 400,
              key: ValueKey("SingleAnimation"),
              child: FlareActor(
                PackageConfig.PACKAGE_ADDRESS + "assets/flare/addSomeBlock.flr",
                animation: "animation",
              ),
            ),
            Text(
              "Add some block!!!",
              key: ValueKey("SingleText"),
              style: TextStyle(fontSize: 30, fontWeight: FontWeight.bold),
            ),
            SizedBox(
              key: ValueKey("SingleSizedBox"),
              height: 20,
            ),
            _buildAddButton(-1,
                key: ValueKey("SingleAddButton"), axis: Axis.horizontal)
          ]
        ],
      ),
    );
  }

  /// Build Every Draggable list item
  _buildListItem(int i, Widget child) {
    return Column(
      key: ValueKey(_list[i]),
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        MouseRegion(
          onHover: (e) {
            if (_addButtonHoveredIndex != i)
              setState(() {
                _addButtonHoveredIndex = i;
              });
          },
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisSize: MainAxisSize.min,
            children: [
              Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  MouseRegion(
                    cursor: cursor,
                    child: AddBlockItem(
                      iconData: Icons.menu,
                      iconColor: Colors.black54,
                    ),
                  ),
                  if (widget.canDeleteBlocks)
                    AddBlockItem(
                      iconData: Icons.delete,
                      iconColor: Colors.red,
                      onTap: () {
                        setState(() {
                          _list.removeAt(i);

                          if (widget.controller != null)
                            widget.controller.bloxItems = _list;
                        });
                      },
                    ),
                  if (_addButtonHoveredIndex == i)
                    MouseRegion(
                        onExit: (e) {
                          setState(() {
                            _addButtonHoveredIndex = -1;
                          });
                        },
                        child: _buildAddButton(i)),
                ],
              ),
              SizedBox(
                width: 10,
              ),
              Expanded(
                child: Container(
                  padding: EdgeInsets.all(16),
                  decoration: BoxDecoration(
                    color: Colors.white,
                    boxShadow: [
                      BoxShadow(
                          color: Colors.black12,
                          blurRadius: 10,
                          spreadRadius: 0.5)
                    ],
                    borderRadius: BorderRadius.circular(widget.borderRadius),
                  ),
                  child: child,
                ),
              ),
            ],
          ),
        ),
        SizedBox(
          height: 10,
        )
      ],
    );
  }

  /// Add Section Button Options
  _buildAddButton(int index, {Key key, Axis axis = Axis.vertical}) {
    return AddBlockWidget(
      key: key,
      axis: axis,
      openOnHover: widget.addBlockButtonOpenOnHover,
      builder: (display, children) {
        display(false);
        return [
          /// Title
          AddBlockItem(
            onTap: () {
              setState(() {
                _list.insert(
                    index + 1,
                    TextBlock(
                        index + 1,
                        "",
                        TextBlockAttr(Constants.DEFAULT_FONT_FAMILY, 30, false,
                            color: BlockColor.black(),
                            align: TextBlockAlign.auto)));

                if (widget.controller != null)
                  widget.controller.bloxItems = _list;
              });
            },
            iconData: Icons.title,
            iconColor: Colors.blue,
            iconSize: 18,
          ),

          /// Paragraph
          AddBlockItem(
            onTap: () {
              setState(() {
                _list.insert(
                    index + 1,
                    TextBlock(
                        index + 1,
                        "",
                        TextBlockAttr(Constants.DEFAULT_FONT_FAMILY, 16, true,
                            color: BlockColor.black(),
                            align: TextBlockAlign.auto)));

                if (widget.controller != null)
                  widget.controller.bloxItems = _list;
              });
            },
            iconData: Icons.format_align_right,
            iconColor: Colors.purple,
            iconSize: 18,
          ),

          /// Image
          AddBlockItem(
            onTap: () {
              String defaultImage = widget.imageEditor(null).defaultImageUrl;
              setState(() {
                _list.insert(
                  index + 1,
                  ImageBlock(index + 1,
                      ImageBlockAttr(defaultImage, "Image", ""), defaultImage),
                );

                if (widget.controller != null)
                  widget.controller.bloxItems = _list;
              });
            },
            iconData: Icons.insert_photo,
            iconColor: Colors.green[600],
            iconSize: 18,
          ),
        ];
      },
    );
  }
}

class BloxController {
  List<Block> bloxItems = [];

  BloxController({bloxItems}) {
    this.bloxItems = bloxItems ?? [];
  }
}
